"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const js_client_sdk_common_1 = require("@eppo/js-client-sdk-common");
const util_1 = require("../util");
class FileBackedNamedEventQueue {
    constructor(name) {
        this.name = name;
        this.eventKeys = [];
        this.queueDirectory = path.resolve(process.cwd(), `.queues/${this.name}`);
        this.metadataFile = path.join(this.queueDirectory, 'metadata.json');
        if (!fs.existsSync(this.queueDirectory)) {
            fs.mkdirSync(this.queueDirectory, { recursive: true });
        }
        this.loadStateFromFile();
    }
    splice(count) {
        const arr = Array.from({ length: count }, () => this.shift());
        return (0, util_1.takeWhile)(arr, (item) => item !== undefined);
    }
    isEmpty() {
        return this.length === 0;
    }
    get length() {
        return this.eventKeys.length;
    }
    push(event) {
        const eventKey = this.generateEventKey(event);
        const eventFilePath = this.getEventFilePath(eventKey);
        fs.writeFileSync(eventFilePath, JSON.stringify(event), 'utf8');
        this.eventKeys.push(eventKey);
        this.saveStateToFile();
    }
    *[Symbol.iterator]() {
        for (const key of this.eventKeys) {
            const eventFilePath = this.getEventFilePath(key);
            if (fs.existsSync(eventFilePath)) {
                const eventData = fs.readFileSync(eventFilePath, 'utf8');
                yield JSON.parse(eventData);
            }
        }
    }
    shift() {
        if (this.isEmpty()) {
            return;
        }
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const eventKey = this.eventKeys.shift();
        const eventFilePath = this.getEventFilePath(eventKey);
        if (fs.existsSync(eventFilePath)) {
            const eventData = fs.readFileSync(eventFilePath, 'utf8');
            fs.unlinkSync(eventFilePath);
            this.saveStateToFile();
            return JSON.parse(eventData);
        }
    }
    loadStateFromFile() {
        if (fs.existsSync(this.metadataFile)) {
            try {
                const metadata = fs.readFileSync(this.metadataFile, 'utf8');
                this.eventKeys = JSON.parse(metadata);
            }
            catch (_a) {
                js_client_sdk_common_1.applicationLogger.error('Failed to parse metadata file. Initializing empty queue.');
                this.eventKeys = [];
            }
        }
    }
    saveStateToFile() {
        fs.writeFileSync(this.metadataFile, JSON.stringify(this.eventKeys), 'utf8');
    }
    generateEventKey(event) {
        return this.hashEvent(event);
    }
    getEventFilePath(eventKey) {
        return path.join(this.queueDirectory, `${eventKey}.json`);
    }
    hashEvent(event) {
        const value = JSON.stringify(event);
        let hash = 0;
        for (let i = 0; i < value.length; i++) {
            hash = (hash << 5) - hash + value.charCodeAt(i);
            hash |= 0; // Convert to 32bit integer
        }
        return hash.toString(36);
    }
}
exports.default = FileBackedNamedEventQueue;
//# sourceMappingURL=file-backed-named-event-queue.js.map