"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstance = exports.init = exports.NO_OP_EVENT_DISPATCHER = exports.EppoAssignmentLogger = void 0;
const js_client_sdk_common_1 = require("@eppo/js-client-sdk-common");
const file_backed_named_event_queue_1 = require("./events/file-backed-named-event-queue");
const sdk_data_1 = require("./sdk-data");
const util_1 = require("./util");
const index_1 = require("./util/index");
var js_client_sdk_common_2 = require("@eppo/js-client-sdk-common");
Object.defineProperty(exports, "EppoAssignmentLogger", { enumerable: true, get: function () { return js_client_sdk_common_2.EppoAssignmentLogger; } });
let clientInstance;
exports.NO_OP_EVENT_DISPATCHER = {
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    attachContext: () => { },
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    dispatch: () => { },
};
/**
 * Initializes the Eppo client with configuration parameters.
 * This method should be called once on application startup.
 * After invocation of this method, the SDK will poll Eppo API at regular intervals to retrieve assignment configurations.
 * @param config client configuration
 * @public
 */
async function init(config) {
    const { apiKey, baseUrl, requestTimeoutMs, numInitialRequestRetries, numPollRequestRetries, pollingIntervalMs, throwOnFailedInitialization = true, pollAfterFailedInitialization = false, eventTracking, } = config;
    if (!apiKey) {
        throw new Error('API key is required');
    }
    const configurationRequestParameters = {
        apiKey,
        sdkName: sdk_data_1.sdkName,
        sdkVersion: sdk_data_1.sdkVersion,
        baseUrl,
        requestTimeoutMs,
        numInitialRequestRetries,
        numPollRequestRetries,
        // For server-side, we always want to keep polling for the life of the process
        pollAfterSuccessfulInitialization: true,
        pollAfterFailedInitialization,
        pollingIntervalMs,
        throwOnFailedInitialization,
    };
    const flagConfigurationStore = new js_client_sdk_common_1.MemoryOnlyConfigurationStore();
    const banditVariationConfigurationStore = new js_client_sdk_common_1.MemoryOnlyConfigurationStore();
    const banditModelConfigurationStore = new js_client_sdk_common_1.MemoryOnlyConfigurationStore();
    const eventDispatcher = newEventDispatcher(apiKey, eventTracking);
    clientInstance = new js_client_sdk_common_1.EppoClient({
        flagConfigurationStore,
        banditVariationConfigurationStore,
        banditModelConfigurationStore,
        configurationRequestParameters,
        eventDispatcher,
    });
    clientInstance.setAssignmentLogger(config.assignmentLogger);
    if (config.banditLogger) {
        clientInstance.setBanditLogger(config.banditLogger);
    }
    // default to LRU cache with 50_000 entries.
    // we estimate this will use no more than 10 MB of memory
    // and should be appropriate for most server-side use cases.
    clientInstance.useLRUInMemoryAssignmentCache(50000);
    clientInstance.useExpiringInMemoryBanditAssignmentCache(50000);
    // Fetch configurations (which will also start regular polling per requestConfiguration)
    await clientInstance.fetchFlagConfigurations();
    // Monkey patch the function to use a generated salt if none is provided
    const originalGetPrecomputedConfiguration = clientInstance.getPrecomputedConfiguration;
    clientInstance.getPrecomputedConfiguration = (subjectKey, subjectAttributes = {}, banditActions = {}, salt) => {
        return originalGetPrecomputedConfiguration.call(clientInstance, subjectKey, subjectAttributes, banditActions, salt !== null && salt !== void 0 ? salt : (0, util_1.generateSalt)());
    };
    return clientInstance;
}
exports.init = init;
/**
 * Used to access a singleton SDK client instance.
 * Use the method after calling init() to initialize the client.
 * @returns a singleton client instance or throws an Error if init() has not been called
 */
function getInstance() {
    if (!clientInstance) {
        throw Error('Expected init() to be called to initialize a client instance');
    }
    return clientInstance;
}
exports.getInstance = getInstance;
function newEventDispatcher(sdkKey, config = {}) {
    const { batchSize = 1000, deliveryIntervalMs = 10000, enabled = false, maxQueueSize = 10000, maxRetries = 3, maxRetryDelayMs = 30000, retryIntervalMs = 5000, } = config;
    if (!enabled) {
        return exports.NO_OP_EVENT_DISPATCHER;
    }
    let eventQueue;
    try {
        // Check if the file system is read-only
        if ((0, index_1.isReadOnlyFs)(`${process.cwd()}/.queues`)) {
            js_client_sdk_common_1.applicationLogger.warn('File system appears to be read-only. Using in-memory event queue instead.');
            eventQueue = new js_client_sdk_common_1.BoundedEventQueue('events', [], maxQueueSize);
        }
        else {
            eventQueue = new file_backed_named_event_queue_1.default('events');
        }
    }
    catch (error) {
        // If there's any error during the check, fall back to BoundedEventQueue
        js_client_sdk_common_1.applicationLogger.warn(`Error checking file system: ${error}. Using in-memory event queue instead.`);
        eventQueue = new js_client_sdk_common_1.BoundedEventQueue('events', [], maxQueueSize);
    }
    const emptyNetworkStatusListener = 
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    { isOffline: () => false, onNetworkStatusChange: () => { } };
    return (0, js_client_sdk_common_1.newDefaultEventDispatcher)(eventQueue, emptyNetworkStatusListener, sdkKey, batchSize, {
        deliveryIntervalMs,
        retryIntervalMs,
        maxRetryDelayMs,
        maxRetries,
    });
}
//# sourceMappingURL=index.js.map